@@ -1,37 +1,19 @@
+import json
 import os
-from functools import wraps
+from copy import deepcopy
 
 import requests
-from requests.exceptions import HTTPError
+from requests import Response, HTTPError, JSONDecodeError
 
-from proteus.config import config
-from proteus.logger import logger
-
-
-def _refresh_authentication():
-    def refresh_authentication_if_authenticated(fn):
-        @wraps(fn)
-        def wrapped(*args, **kwargs):
-            try:
-                return fn(*args, **kwargs)
-            except HTTPError as error:
-                logger.error(error.response.content)
-                if error.response.status_code == 401:
-                    global auth
-                    auth.do_login()
-
-                    return fn(*args, **kwargs)
-
-                raise error
-
-        return wrapped
-
-    return refresh_authentication_if_authenticated
+from proteus.config import Config
 
 
 class API:
-    def __init__(self, auth):
+    def __init__(self, auth, config: Config, logger):
         self.auth = auth
+        self.config = deepcopy(config or Config())
+        self.host = config.api_host
+        self.logger = logger
 
     def get(self, url, headers=None, stream=False, **query_args):
         headers = {
@@ -39,9 +21,9 @@ class API:
             "Content-Type": "application/json",
             **(headers or {}),
         }
-        url = f"{config.API_HOST}/{url.strip('/')}"
+        url = self.build_url(url)
         response = requests.get(url, headers=headers, params=query_args, stream=stream)
-        response.raise_for_status()
+        self.raise_for_status(response)
         return response
 
     def put(self, url, data, headers=None):
@@ -50,7 +32,7 @@ class API:
             "Content-Type": "application/json",
             **(headers or {}),
         }
-        url = f"{config.API_HOST}/{url.strip('/')}"
+        url = self.build_url(url)
         return requests.put(url, headers=headers, json=data)
 
     def post(self, url, data, headers=None):
@@ -59,7 +41,7 @@ class API:
             "Content-Type": "application/json",
             **(headers or {}),
         }
-        url = f"{config.API_HOST}/{url.strip('/')}"
+        url = self.build_url(url)
         return requests.post(url, headers=headers, json=data)
 
     def delete(self, url, headers={}, **query_args):
@@ -68,9 +50,9 @@ class API:
             "Content-Type": "application/json",
             **headers,
         }
-        url = f"{config.API_HOST}/{url.strip('/')}"
+        url = self.build_url(url)
         response = requests.delete(url, headers=headers, params=query_args)
-        response.raise_for_status()
+        self.raise_for_status(response)
         return response
 
     def _post_files(self, url, files, headers=None):
@@ -78,16 +60,15 @@ class API:
             "Authorization": f"Bearer {self.auth.access_token}",
             **(headers or {}),
         }
-        url = f"{config.API_HOST}/{url.strip('/')}"
+        url = self.build_url(url)
         response = requests.post(url, headers=headers, files=files)
         try:
-            response.raise_for_status()
+            self.raise_for_status(response)
         except Exception as error:
-            logger.error(response.content)
+            self.logger.error(response.content)
             raise error
         return response
 
-    @_refresh_authentication()
     def post_file(self, url, filepath, content=None, modified=None):
         headers = {}
         if modified is not None:
@@ -99,16 +80,10 @@ class API:
         return self.get(url, stream=stream, timeout=timeout, headers={"content-type": "application/octet-stream"})
 
     def store_download(self, url, localpath, localname, stream=False, timeout=60):
-        logger.info(f"Downloading {url} to {os.path.join(localpath)}")
+        self.logger.info(f"Downloading {url} to {os.path.join(localpath)}")
 
         r = self.download(url, stream=stream, timeout=timeout)
 
-        try:
-            r.raise_for_status()
-        except Exception as error:
-            logger.error("Response cannot raise status")
-            raise error
-
         os.makedirs(localpath, exist_ok=True)
         local = localpath
 
@@ -118,6 +93,36 @@ class API:
         with open(local, "wb") as f:
             f.write(r.content)
 
-        logger.info("Download complete")
+        self.logger.info("Download complete")
 
         return r.status_code
+
+    def build_url(self, url):
+        url = f"{self.host}/{url.strip('/')}"
+
+        args = []
+        if self.config.ignore_worker_status:
+            args.append("ignore_status=1")
+
+        if args:
+            args = "&".join(args)
+            if "?" in url:
+                args = "&" + args
+            else:
+                args = "?" + args
+
+        return url + args
+
+    def raise_for_status(self, response: Response):
+        try:
+            response.raise_for_status()
+        except HTTPError as http_error:
+            try:
+                error_detail = response.json()
+                if isinstance(error_detail, dict):
+                    http_error.args = (
+                        f"{http_error.args[0]}. Returned error " f"payload: {json.dumps(error_detail, indent=2)}",
+                    )
+            except JSONDecodeError:
+                pass
+            raise http_error
