@@ -12,7 +12,7 @@ grammar_str = (
     + c_str
     + r"""
 terminals
-number: /\d+(\.\d+)?/;
+number: /\-?\d+(\.\d+)?/;
 array: /\$[a-zA-Z_][a-zA-Z0-9_]*/;
 """
 )
